package cz.fura;

import java.beans.PropertyEditorSupport;
import java.text.MessageFormat;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Description
 *
 * @author Jan Novotn, FG Forrest a.s. (c) 2007
 * @version $Id: $
 */
public class EmailAddressEditor extends PropertyEditorSupport {
	private static final String MSG = "Expected format \"username@server.domain\" or \"Display name (username@server.domain)\", got ${0}!";
	public static final String MSG_FORMAT = "Invalid format of message address: {0}";
	//Set the message pattern string
    private static Pattern p = Pattern.compile(".+@.+\\..+");

	/**
	 * Gets the property value as a string suitable for presentation
	 * to a human to edit.
	 *
	 * @return The property value as a string suitable for presentation
	 *         to a human to edit.
	 *         <p>   Returns "null" is the value can't be expressed as a string.
	 *         <p>   If a non-null value is returned, then the PropertyEditor should
	 *         be prepared to parse that string back in setAsText().
	 */
	public String getAsText() {
		EmailAddress email = (EmailAddress)getValue();
		if (email == null) {
			return "";
		} else if (email.getName() != null) {
			return email.getName() + " (" + email.getAddress() + ")";
		} else {
			return email.getAddress();
		}
	}

	/**
	 * Sets the property value by parsing a given String.  May raise
	 * java.lang.IllegalArgumentException if either the String is
	 * badly formatted or if this kind of property can't be expressed
	 * as text.
	 *
	 * @param text The string to be parsed.
	 */
	public void setAsText(String text) throws IllegalArgumentException {
		if (text == null || text.trim().length() == 0) {
			setValue(null);
		} else {
			int index = text.indexOf("(");
			int endIndex = text.indexOf(")", index);
			if (index > -1) {
				if (endIndex < index) throwException(text);
				String name = text.substring(0, index -1).trim();
				String email = text.substring(index + 1, endIndex).trim();
				validateEmailFormat(email);
				setValue(new EmailAddress(email, name));
			} else {
				String email = text.trim();
				validateEmailFormat(email);
				setValue(new EmailAddress(email));
			}
		}
	}

	/**
	 * Validates format of message.
	 * @param email
	 * @throws IllegalArgumentException
	 */
	private void validateEmailFormat(String email) throws IllegalArgumentException {
	  //Match the given string with the pattern
      Matcher m = p.matcher(email);
      //check whether match is found
      if (!m.matches())
	  	throw new IllegalArgumentException(
				  MessageFormat.format(MSG_FORMAT, new Object[] {email} )
		  );
	}

	/**
	 * Throws descriptive exception.
	 * @param text
	 * @throws IllegalArgumentException
	 */
	private void throwException(String text) throws IllegalArgumentException {
		throw new IllegalArgumentException(
				MessageFormat.format(MSG, new Object[] {text} )
		);
	}

}